package ch.ethz.se.rpnc.instruct;

import java.util.Hashtable;

import ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitor;
import ch.ethz.se.rpnc.ui.ResourceManager;

public class CalculatorInstructOperator implements ICalculatorInstruct {
	
	// Type of operator.
	private CalculatorOperatorType type;
	
	// Private constructor to implement Multi-ton pattern.
	private CalculatorInstructOperator(CalculatorOperatorType t){
		setType(t);
	}
	
	// A pool of operator object, one of each operator type.
	protected static Hashtable<CalculatorOperatorType, CalculatorInstructOperator> operators;
	static{
		operators = new Hashtable<CalculatorOperatorType, CalculatorInstructOperator>();
		for(CalculatorOperatorType t: CalculatorOperatorType.values()){
			operators.put(t, new CalculatorInstructOperator(t));
		}
	}
	
	/** 
	 * Get instruct operator according to its type.
	 */
	public static CalculatorInstructOperator getInstructOperator(CalculatorOperatorType t){
		return operators.get(t);
	}
	
	/**
	 * Get instruct operator according to its text representation.
	 */
	public static CalculatorInstructOperator getInstructOperator(String text){
		for (CalculatorOperatorType ot : CalculatorOperatorType.values()) {
			if (text.equals(ot.getSymbol())) {
				return CalculatorInstructOperator.getInstructOperator(ot);
			}
		}
		return null;
	}

	@Override
	public String getInstructText(){
		return getType().getSymbol();
	}
	
	@Override
	public void accept(CalculatorInstructVisitor v) throws Exception {
		v.visitOperator(this);
	}

	/**
	 * Get type of operator.
	 * @return Operator type.
	 */
	public CalculatorOperatorType getType() {
		return type;
	}

	// Set type of operator.
	private void setType(CalculatorOperatorType type) {
		this.type = type;
	}

}
