/**
 * 
 */
package ch.ethz.se.rpnc.node.visitor;

import java.util.EmptyStackException;
import java.util.Stack;

import ch.ethz.se.rpnc.exception.IllegalNumberOfOperandsException;
import ch.ethz.se.rpnc.node.DivideByOperator;
import ch.ethz.se.rpnc.node.MinusOperator;
import ch.ethz.se.rpnc.node.Node;
import ch.ethz.se.rpnc.node.Operand;
import ch.ethz.se.rpnc.node.PlusOperator;
import ch.ethz.se.rpnc.node.TimesOperator;
import ch.ethz.se.rpnc.ui.ResourceManager;
import ch.ethz.se.rpnc.ui.UIMessage;

/**
 * Evaluates expressions.
 * 
 * @author aleitner
 */
public class Evaluator implements Visitor {

	/**
	 * Last evaluation result
	 */
	private double result;

	/**
	 * Stack to evaluate on
	 */
	private Stack<Node> stack;

	/**
	 * Create new evaluator.
	 * 
	 * @param s
	 *            Stack to evaluate expressions on.
	 */
	public Evaluator(Stack<Node> s) {
		stack = s;
	}

	/**
	 * Last evaluation result.
	 * 
	 * @return Last evaluation result.
	 */
	public double getResult() {
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.ethz.se.rpnc2.Visitor#visitOperand(ch.ethz.se.rpnc2.Operand)
	 */
	public void visitOperand(Operand o) {
		result = o.getValue();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * ch.ethz.se.rpnc2.Visitor#visitPlusOperator(ch.ethz.se.rpnc2.PlusOperator)
	 */
	public void visitPlusOperator(PlusOperator o) throws Exception {
		double op1 = nextOperand();
		double op2 = nextOperand();
		result = op1 + op2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * ch.ethz.se.rpnc2.Visitor#visitMinusOperator(ch.ethz.se.rpnc2.MinusOperator
	 * )
	 */
	public void visitMinusOperator(MinusOperator o) throws Exception {
		double op1 = nextOperand();
		double op2 = nextOperand();
		result = op2 - op1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * ch.ethz.se.rpnc2.Visitor#visitTimesOperator(ch.ethz.se.rpnc2.TimesOperator
	 * )
	 */
	public void visitTimesOperator(TimesOperator o) throws Exception {
		double op1 = nextOperand();
		double op2 = nextOperand();
		result = op1 * op2;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seech.ethz.se.rpnc2.Visitor#visitDivideByOperator(ch.ethz.se.rpnc2.
	 * DivideByOperator)
	 */
	public void visitDivideByOperator(DivideByOperator o) throws Exception {
		double op1 = nextOperand();
		double op2 = nextOperand();
		result = op2 / op1;
	}

	private double nextOperand() throws Exception {
		try {
			Node n = stack.pop();
			n.accept(this);
			return result;
		} catch (EmptyStackException e) {
			throw new IllegalNumberOfOperandsException(
					ResourceManager
					.getLocalizedUIMessage(UIMessage.MSG_MISSING_OPERAND));
		}
	}

}
